@@ -47,4 +47,8 @@ public interface Constants extends Messages {
     String drag_component_name_table();
 
     String drag_component_name_filter();
+
+    String externalComponentsGroupName();
+
+    String internalComponentsGroupName();
 }
