@@ -17,8 +17,12 @@ package org.guvnor.rest.backend;
 
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Locale;
+import java.util.Map;
 import java.util.concurrent.atomic.AtomicLong;
+import java.util.stream.Collectors;
 
 import javax.annotation.security.RolesAllowed;
 import javax.enterprise.context.ApplicationScoped;
@@ -27,12 +31,15 @@ import javax.inject.Named;
 import javax.ws.rs.Consumes;
 import javax.ws.rs.DELETE;
 import javax.ws.rs.GET;
+import javax.ws.rs.HeaderParam;
 import javax.ws.rs.POST;
+import javax.ws.rs.PUT;
 import javax.ws.rs.Path;
 import javax.ws.rs.PathParam;
 import javax.ws.rs.Produces;
 import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.Context;
+import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.Response;
 import javax.ws.rs.core.Response.Status;
@@ -41,6 +48,9 @@ import javax.ws.rs.core.Variant;
 
 import org.guvnor.common.services.project.model.WorkspaceProject;
 import org.guvnor.common.services.project.service.WorkspaceProjectService;
+import org.guvnor.rest.client.AddBranchJobRequest;
+import org.guvnor.rest.client.AddBranchRequest;
+import org.guvnor.rest.client.BranchResponse;
 import org.guvnor.rest.client.CloneProjectJobRequest;
 import org.guvnor.rest.client.CloneProjectRequest;
 import org.guvnor.rest.client.CompileProjectRequest;
@@ -53,24 +63,27 @@ import org.guvnor.rest.client.JobRequest;
 import org.guvnor.rest.client.JobResult;
 import org.guvnor.rest.client.JobStatus;
 import org.guvnor.rest.client.ProjectResponse;
+import org.guvnor.rest.client.RemoveBranchJobRequest;
 import org.guvnor.rest.client.RemoveSpaceRequest;
 import org.guvnor.rest.client.Space;
 import org.guvnor.rest.client.SpaceRequest;
 import org.guvnor.rest.client.TestProjectRequest;
 import org.guvnor.structure.organizationalunit.OrganizationalUnit;
 import org.guvnor.structure.organizationalunit.OrganizationalUnitService;
+import org.guvnor.structure.repositories.Branch;
 import org.guvnor.structure.repositories.PublicURI;
 import org.kie.soup.commons.validation.PortablePreconditions;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.uberfire.io.IOService;
+import org.uberfire.rpc.SessionInfo;
 import org.uberfire.spaces.SpacesAPI;
 
 import static org.guvnor.rest.backend.PermissionConstants.REST_PROJECT_ROLE;
 import static org.guvnor.rest.backend.PermissionConstants.REST_ROLE;
 
 /**
- * REST services
+ * REST services for project management related operations
  */
 @Path("/")
 @Named
@@ -78,7 +91,13 @@ import static org.guvnor.rest.backend.PermissionConstants.REST_ROLE;
 public class ProjectResource {
 
     private static final Logger logger = LoggerFactory.getLogger(ProjectResource.class);
-    private static Variant defaultVariant = Variant.mediaTypes(MediaType.APPLICATION_JSON_TYPE).add().build().get(0);
+    private static final String ACCEPT_LANGUAGE = "acceptLanguage";
+    private Variant defaultVariant = getDefaultVariant();
+
+    protected Variant getDefaultVariant() {
+        return Variant.mediaTypes(MediaType.APPLICATION_JSON_TYPE).add().build().get(0);
+    }
+
     @Context
     protected UriInfo uriInfo;
     @Inject
@@ -89,13 +108,13 @@ public class ProjectResource {
     @Inject
     private WorkspaceProjectService workspaceProjectService;
     @Inject
-    private SpacesAPI spaces;
-    @Inject
     private JobRequestScheduler jobRequestObserver;
     @Inject
     private JobResultManager jobManager;
     @Inject
     private SpacesAPI spacesAPI;
+    @Inject
+    private SessionInfo sessionInfo;
 
     private AtomicLong counter = new AtomicLong(0);
 
@@ -103,6 +122,7 @@ public class ProjectResource {
         JobResult jobResult = new JobResult();
         jobResult.setJobId(jobId);
         jobResult.setStatus(JobStatus.ACCEPTED);
+        jobManager.putJob(jobResult);
     }
 
     @GET
@@ -180,6 +200,7 @@ public class ProjectResource {
     @RolesAllowed({REST_ROLE, REST_PROJECT_ROLE})
     public Response createProject(
             @PathParam("spaceName") String spaceName,
+            @HeaderParam(HttpHeaders.ACCEPT_LANGUAGE) Locale locales,
             CreateProjectRequest createProjectRequest) {
         logger.debug("-----createProject--- , spaceName: {} , project name: {}",
                      spaceName,
@@ -188,7 +209,9 @@ public class ProjectResource {
         assertObjectExists(organizationalUnitService.getOrganizationalUnit(spaceName),
                            "space",
                            spaceName);
-
+        
+        final Map<String, Object> headers = new HashMap<>();
+        headers.put(ACCEPT_LANGUAGE, locales);
         final String id = newId();
         final CreateProjectJobRequest jobRequest = new CreateProjectJobRequest();
         jobRequest.setStatus(JobStatus.ACCEPTED);
@@ -200,7 +223,8 @@ public class ProjectResource {
         jobRequest.setDescription(createProjectRequest.getDescription());
         addAcceptedJobResult(id);
 
-        jobRequestObserver.createProjectRequest(jobRequest);
+        jobRequestObserver.createProjectRequest(jobRequest,
+                                                headers);
 
         return createAcceptedStatusResponse(jobRequest);
     }
@@ -285,6 +309,126 @@ public class ProjectResource {
         return projectResponse;
     }
 
+    @GET
+    @Produces(MediaType.APPLICATION_JSON)
+    @Path("/spaces/{spaceName}/projects/{projectName}/branches")
+    @RolesAllowed({REST_ROLE, REST_PROJECT_ROLE})
+    public Collection<BranchResponse> getBranches(@PathParam("spaceName") String spaceName,
+                                                  @PathParam("projectName") String projectName) {
+
+        logger.debug("-----getBranches---, space name: {}, project name: {}",
+                     spaceName,
+                     projectName);
+
+        assertObjectExists(organizationalUnitService.getOrganizationalUnit(spaceName),
+                           "space",
+                           spaceName);
+
+        final WorkspaceProject workspaceProject = workspaceProjectService.resolveProject(
+                spacesAPI.getSpace(spaceName),
+                projectName);
+
+        assertObjectExists(workspaceProject,
+                           "project",
+                           projectName);
+
+        return workspaceProject
+                .getRepository()
+                .getBranches()
+                .stream()
+                .map(this::getBranchResponse)
+                .collect(Collectors.toList());
+    }
+
+    @POST
+    @Consumes(MediaType.APPLICATION_JSON)
+    @Produces(MediaType.APPLICATION_JSON)
+    @Path("/spaces/{spaceName}/projects/{projectName}/branches")
+    @RolesAllowed({REST_ROLE, REST_PROJECT_ROLE})
+    public Response addBranch(@PathParam("spaceName") String spaceName,
+                              @PathParam("projectName") String projectName,
+                              AddBranchRequest addBranchRequest) {
+
+        logger.debug("-----addBranch--- , spaceName: {} , project name: {}, branch Name: {}",
+                     spaceName,
+                     projectName,
+                     addBranchRequest.getNewBranchName());
+
+        assertObjectExists(organizationalUnitService.getOrganizationalUnit(spaceName),
+                           "space",
+                           spaceName);
+
+        final WorkspaceProject workspaceProject = workspaceProjectService.resolveProject(
+                spacesAPI.getSpace(spaceName),
+                projectName);
+
+        assertObjectExists(workspaceProject,
+                           "project",
+                           projectName);
+
+        final String id = newId();
+        final AddBranchJobRequest jobRequest = new AddBranchJobRequest();
+        jobRequest.setStatus(JobStatus.ACCEPTED);
+        jobRequest.setJobId(id);
+        jobRequest.setSpaceName(spaceName);
+        jobRequest.setProjectName(projectName);
+        jobRequest.setNewBranchName(addBranchRequest.getNewBranchName());
+        jobRequest.setBaseBranchName(addBranchRequest.getBaseBranchName());
+        jobRequest.setUserIdentifier(sessionInfo.getIdentity().getIdentifier());
+        addAcceptedJobResult(id);
+
+        jobRequestObserver.addBranchRequest(jobRequest);
+
+        return createAcceptedStatusResponse(jobRequest);
+    }
+
+    @DELETE
+    @Consumes(MediaType.APPLICATION_JSON)
+    @Produces(MediaType.APPLICATION_JSON)
+    @Path("/spaces/{spaceName}/projects/{projectName}/branches/{branchName}")
+    @RolesAllowed({REST_ROLE, REST_PROJECT_ROLE})
+    public Response removeBranch(@PathParam("spaceName") String spaceName,
+                                 @PathParam("projectName") String projectName,
+                                 @PathParam("branchName") String branchName) {
+
+        logger.debug("-----removeBranch--- , spaceName: {} , project name: {}, branch Name: {}",
+                     spaceName,
+                     projectName,
+                     branchName);
+
+        assertObjectExists(organizationalUnitService.getOrganizationalUnit(spaceName),
+                           "space",
+                           spaceName);
+
+        final WorkspaceProject workspaceProject = workspaceProjectService.resolveProject(
+                spacesAPI.getSpace(spaceName),
+                projectName);
+
+        assertObjectExists(workspaceProject,
+                           "project",
+                           projectName);
+
+        final String id = newId();
+        final RemoveBranchJobRequest jobRequest = new RemoveBranchJobRequest();
+        jobRequest.setStatus(JobStatus.ACCEPTED);
+        jobRequest.setJobId(id);
+        jobRequest.setSpaceName(spaceName);
+        jobRequest.setProjectName(projectName);
+        jobRequest.setBranchName(branchName);
+        jobRequest.setUserIdentifier(sessionInfo.getIdentity().getIdentifier());
+        addAcceptedJobResult(id);
+
+        jobRequestObserver.removeBranchRequest(jobRequest);
+
+        return createAcceptedStatusResponse(jobRequest);
+    }
+
+    private BranchResponse getBranchResponse(Branch branch) {
+        final BranchResponse branchResponse = new BranchResponse();
+        branchResponse.setName(branch.getName());
+        return branchResponse;
+    }
+
     private ProjectResponse getProjectResponse(WorkspaceProject workspaceProject) {
         final ProjectResponse projectResponse = new ProjectResponse();
         projectResponse.setName(workspaceProject.getName());
@@ -316,9 +460,25 @@ public class ProjectResource {
     public Response compileProject(
             @PathParam("spaceName") String spaceName,
             @PathParam("projectName") String projectName) {
-        logger.debug("-----compileProject--- , space name: {}, project name: {}",
+
+        return compileProject(spaceName,
+                              projectName,
+                              null);
+    }
+
+    @POST
+    @Produces(MediaType.APPLICATION_JSON)
+    @Path("/spaces/{spaceName}/projects/{projectName}/branches/{branchName}/maven/compile")
+    @RolesAllowed({REST_ROLE, REST_PROJECT_ROLE})
+    public Response compileProject(
+            @PathParam("spaceName") String spaceName,
+            @PathParam("projectName") String projectName,
+            @PathParam("branchName") String branchName) {
+
+        logger.debug("-----compileProject--- , space name: {}, project name: {}, branch name: {}",
                      spaceName,
-                     projectName);
+                     projectName,
+                     branchName);
 
         final String id = newId();
         final CompileProjectRequest jobRequest = new CompileProjectRequest();
@@ -326,6 +486,7 @@ public class ProjectResource {
         jobRequest.setJobId(id);
         jobRequest.setProjectName(projectName);
         jobRequest.setSpaceName(spaceName);
+        jobRequest.setBranchName(branchName);
         addAcceptedJobResult(id);
 
         jobRequestObserver.compileProjectRequest(jobRequest);
@@ -340,8 +501,24 @@ public class ProjectResource {
     public Response installProject(
             @PathParam("spaceName") String spaceName,
             @PathParam("projectName") String projectName) {
-        logger.debug("-----installProject--- , project name: {}",
-                     projectName);
+
+        return installProject(spaceName,
+                              projectName,
+                              null);
+    }
+
+    @POST
+    @Produces(MediaType.APPLICATION_JSON)
+    @Path("/spaces/{spaceName}/projects/{projectName}/branches/{branchName}/maven/install")
+    @RolesAllowed({REST_ROLE, REST_PROJECT_ROLE})
+    public Response installProject(
+            @PathParam("spaceName") String spaceName,
+            @PathParam("projectName") String projectName,
+            @PathParam("branchName") String branchName) {
+
+        logger.debug("-----installProject--- , project name: {}, branch name: {}",
+                     projectName,
+                     branchName);
 
         PortablePreconditions.checkNotNull("spaceName", spaceName);
         PortablePreconditions.checkNotNull("projectName", projectName);
@@ -352,6 +529,7 @@ public class ProjectResource {
         jobRequest.setJobId(id);
         jobRequest.setSpaceName(spaceName);
         jobRequest.setProjectName(projectName);
+        jobRequest.setBranchName(branchName);
         addAcceptedJobResult(id);
 
         jobRequestObserver.installProjectRequest(jobRequest);
@@ -367,8 +545,25 @@ public class ProjectResource {
     public Response testProject(
             @PathParam("spaceName") String spaceName,
             @PathParam("projectName") String projectName) {
-        logger.debug("-----testProject--- , project name: {}",
-                     projectName);
+
+        return testProject(spaceName,
+                           projectName,
+                           null);
+    }
+
+    @POST
+    @Consumes(MediaType.APPLICATION_JSON)
+    @Produces(MediaType.APPLICATION_JSON)
+    @Path("/spaces/{spaceName}/projects/{projectName}/branches/{branchName}/maven/test")
+    @RolesAllowed({REST_ROLE, REST_PROJECT_ROLE})
+    public Response testProject(
+            @PathParam("spaceName") String spaceName,
+            @PathParam("projectName") String projectName,
+            @PathParam("branchName") String branchName) {
+
+        logger.debug("-----testProject--- , project name: {}, branch name: {}",
+                     projectName,
+                     branchName);
 
         final String id = newId();
         final TestProjectRequest jobRequest = new TestProjectRequest();
@@ -376,6 +571,7 @@ public class ProjectResource {
         jobRequest.setJobId(id);
         jobRequest.setProjectName(projectName);
         jobRequest.setSpaceName(spaceName);
+        jobRequest.setBranchName(branchName);
         addAcceptedJobResult(id);
 
         jobRequestObserver.testProjectRequest(jobRequest);
@@ -390,8 +586,24 @@ public class ProjectResource {
     public Response deployProject(
             @PathParam("spaceName") String spaceName,
             @PathParam("projectName") String projectName) {
-        logger.debug("-----deployProject--- , project name: {}",
-                     projectName);
+
+        return deployProject(spaceName,
+                             projectName,
+                             null);
+    }
+
+    @POST
+    @Produces(MediaType.APPLICATION_JSON)
+    @Path("/spaces/{spaceName}/projects/{projectName}/branches/{branchName}/maven/deploy")
+    @RolesAllowed({REST_ROLE, REST_PROJECT_ROLE})
+    public Response deployProject(
+            @PathParam("spaceName") String spaceName,
+            @PathParam("projectName") String projectName,
+            @PathParam("branchName") String branchName) {
+
+        logger.debug("-----deployProject--- , project name: {}, branch name: {}",
+                     projectName,
+                     branchName);
 
         final String id = newId();
         final DeployProjectRequest jobRequest = new DeployProjectRequest();
@@ -399,6 +611,7 @@ public class ProjectResource {
         jobRequest.setJobId(id);
         jobRequest.setProjectName(projectName);
         jobRequest.setSpaceName(spaceName);
+        jobRequest.setBranchName(branchName);
         addAcceptedJobResult(id);
 
         jobRequestObserver.deployProjectRequest(jobRequest);
@@ -424,6 +637,7 @@ public class ProjectResource {
     private Space getSpace(OrganizationalUnit ou) {
         final Space space = new Space();
         space.setName(ou.getName());
+        space.setDescription(ou.getDescription());
         space.setOwner(ou.getOwner());
         space.setDefaultGroupId(ou.getDefaultGroupId());
 
@@ -468,6 +682,7 @@ public class ProjectResource {
         jobRequest.setStatus(JobStatus.ACCEPTED);
         jobRequest.setJobId(id);
         jobRequest.setSpaceName(space.getName());
+        jobRequest.setDescription(space.getDescription());
         jobRequest.setOwner(space.getOwner());
         jobRequest.setDefaultGroupId(space.getDefaultGroupId());
         addAcceptedJobResult(id);
@@ -477,6 +692,31 @@ public class ProjectResource {
         return createAcceptedStatusResponse(jobRequest);
     }
 
+    @PUT
+    @Consumes(MediaType.APPLICATION_JSON)
+    @Produces(MediaType.APPLICATION_JSON)
+    @Path("/spaces")
+    @RolesAllowed({REST_ROLE, REST_PROJECT_ROLE})
+    public Response updateSpace(Space space) {
+        logger.debug("-----updateSpace--- , Space name: {}, Default group id : {}",
+                     space.getName(),
+                     space.getDefaultGroupId());
+
+        final String id = newId();
+        final SpaceRequest jobRequest = new SpaceRequest();
+        jobRequest.setStatus(JobStatus.ACCEPTED);
+        jobRequest.setJobId(id);
+        jobRequest.setSpaceName(space.getName());
+        jobRequest.setDescription(space.getDescription());
+        jobRequest.setOwner(space.getOwner());
+        jobRequest.setDefaultGroupId(space.getDefaultGroupId());
+        addAcceptedJobResult(id);
+
+        jobRequestObserver.updateSpaceRequest(jobRequest);
+
+        return createAcceptedStatusResponse(jobRequest);
+    }
+
     @DELETE
     @Produces(MediaType.APPLICATION_JSON)
     @Path("/spaces/{spaceName}")
@@ -501,16 +741,16 @@ public class ProjectResource {
         return createAcceptedStatusResponse(jobRequest);
     }
 
-    private void assertObjectExists(final Object o,
-                                    final String objectInfo,
-                                    final String objectName) {
+    protected void assertObjectExists(final Object o,
+                                      final String objectInfo,
+                                      final String objectName) {
         if (o == null) {
             throw new WebApplicationException(String.format("Could not find %s with name %s.", objectInfo, objectName),
                                               Response.status(Status.NOT_FOUND).build());
         }
     }
 
-    private Response createAcceptedStatusResponse(final JobRequest jobRequest) {
+    protected Response createAcceptedStatusResponse(final JobRequest jobRequest) {
         return Response.status(Status.ACCEPTED).entity(jobRequest).variant(defaultVariant).build();
     }
 
