@@ -30,7 +30,8 @@ import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.mockito.InjectMocks;
 import org.mockito.Mock;
-import org.mockito.runners.MockitoJUnitRunner;
+import org.mockito.junit.MockitoJUnitRunner;
+import org.uberfire.client.util.GWTEditorNativeRegister;
 import org.uberfire.client.workbench.events.NewPerspectiveEvent;
 import org.uberfire.client.workbench.events.NewWorkbenchScreenEvent;
 import org.uberfire.client.workbench.type.ClientResourceType;
@@ -60,6 +61,9 @@ public class ActivityBeansCacheTest {
     @Mock
     private ExperimentalActivitiesAuthorizationManager experimentalActivitiesAuthorizationManager;
 
+    @Mock
+    private GWTEditorNativeRegister gwtEditorNativeRegister;
+
     @InjectMocks
     ActivityBeansCache cache;
 
@@ -67,11 +71,12 @@ public class ActivityBeansCacheTest {
     public void setUp() {
         resourceTypeManagerCache = new ResourceTypeManagerCache(categoriesManagerCache);
 
-        cache = new ActivityBeansCache(iocManager,
+        cache = spy(new ActivityBeansCache(iocManager,
                                        newPerspectiveEventEvent,
                                        newWorkbenchScreenEvent,
                                        resourceTypeManagerCache,
-                                       experimentalActivitiesAuthorizationManager);
+                                       experimentalActivitiesAuthorizationManager,
+                                       gwtEditorNativeRegister));
     }
 
     @Test
@@ -83,7 +88,21 @@ public class ActivityBeansCacheTest {
 
         assertEquals(cache.getMockDef(),
                      cache.getActivity(cache.getIdMock()));
-        assertTrue(cache.getSplashScreens().contains(cache.getSplashScreenActivity()));
+        assertTrue(cache.getSplashScreens().contains(cache.getActivity()));
+    }
+
+    @Test
+    public void initShouldCacheClientEditors() throws Exception {
+        ActivityBeansCacheUnitTestWrapper cache = spy(new ActivityBeansCacheUnitTestWrapper());
+        cache.mockClientEditorBehaviour();
+
+        cache.init();
+
+        verify(cache).registerGwtClientBean(eq("mockDef1"), any());
+
+        assertEquals(cache.getMockDef(),
+                     cache.getActivity(cache.getIdMock()));
+        assertTrue(cache.getActivitiesById().contains(cache.getActivity().getIdentifier()));
     }
 
     @Test
@@ -191,9 +210,6 @@ public class ActivityBeansCacheTest {
         Collection<SyncBeanDef> resourceTypeBeans1 = createResourceType("MODEL");
         when(iocManager.lookupBeans(eq("resource1"))).thenReturn(resourceTypeBeans1);
 
-        Collection<SyncBeanDef> resourceTypeBeans2 = createResourceType("MODEL");
-        when(iocManager.lookupBeans(eq("resource2"))).thenReturn(resourceTypeBeans2);
-
         SyncBeanDef mock = mock(SyncBeanDef.class);
         when(mock.getName()).thenReturn("resource1");
         cache.addNewEditorActivity(mock,
@@ -212,9 +228,7 @@ public class ActivityBeansCacheTest {
 
     private Collection<SyncBeanDef> createResourceType(String type) {
         Category model = mock(Category.class);
-        when(model.getName()).thenReturn(type);
         ClientResourceType clientResourceType = mock(ClientResourceType.class);
-        when(clientResourceType.getCategory()).thenReturn(model);
         SyncBeanDef<ClientResourceType> syncBeanDef = mock(SyncBeanDef.class);
         when(syncBeanDef.getInstance()).thenReturn(clientResourceType);
         return Arrays.asList(syncBeanDef);
@@ -253,4 +267,10 @@ public class ActivityBeansCacheTest {
         assertEquals(perspectiveActivities.size(),
                      1);
     }
+
+    @Test
+    public void getActivitiesNull() {
+        assertNull(cache.getActivity((String) null));
+    }
+
 }
